// View.h : interface of the CKrbManagerView class
//
/////////////////////////////////////////////////////////////////////////////

class CKrbManagerDoc;

class CKrbManagerView : public CListView
{
protected: // create from serialization only
	CKrbManagerView();
	DECLARE_DYNCREATE(CKrbManagerView)

// Attributes
public:
	CKrbManagerDoc* GetDocument();
    CImageList image_list;

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CKrbManagerView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	protected:
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CKrbManagerView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
    void AddColumn(int index, LPSTR plsz, int width = -1);
    void UpdateCacheList();

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CKrbManagerView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg LRESULT OnUpdateCache(WPARAM wParam, LPARAM lParam);
	afx_msg void OnFileDestroy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in View.cpp
inline CKrbManagerDoc* CKrbManagerView::GetDocument()
   { return (CKrbManagerDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
