LIBRARY	krb	BASE=0x07000000
EXPORTS
	krb_get_err_text
	
	newTktMem
	getTktMem
	firstCred
	nextCredIndex
	currCredIndex
	nextFreeIndex
	
	k_flock
	k_localtime
	k_getsockinst
	k_getportbyname
	k_get_all_addrs

	krb_equiv

	afs_string_to_key

	krb_life_to_time
	krb_time_to_life
	krb_life_to_atime
	krb_atime_to_life

	tf_get_cred
	tf_get_pinst
	tf_get_pname
	tf_put_pinst
	tf_put_pname
	tf_init
	tf_create
	tf_save_cred
	tf_close

	krb_mk_priv
	krb_rd_priv

	create_auth_reply
	krb_get_phost
	krb_realmofhost
	tkt_string
	create_ciph
	decomp_ticket
	dest_tkt
	get_ad_tkt
	in_tkt
	k_gethostname
	k_isinst
	k_isname
	k_isrealm
	kname_parse
	krb_parse_name
	krb_unparse_name
	krb_unparse_name_long
	krb_create_ticket
	krb_get_admhst
	krb_get_cred
	krb_get_in_tkt
	krb_get_krbhst
	krb_get_lrealm
	krb_get_default_realm
	krb_get_pw_in_tkt
	krb_get_svc_in_tkt
	krb_get_tf_fullname
	krb_get_tf_realm
	krb_kntoln
	krb_mk_req
	krb_net_read
	krb_net_write
	krb_rd_err
	krb_rd_req
	krb_recvauth
	krb_sendauth
	krb_set_key
	krb_set_lifetime 
	read_service_key
	save_credentials
	send_to_kdc
	krb_mk_err
	krb_mk_safe
	krb_rd_safe
	ad_print
	cr_err_reply
	krb_set_tkt_string
	krb_get_default_principal
	krb_realm_parse
	krb_verify_user
	kset_logfile
	getst
