LIBRARY kclnt32
EXPORTS
	KClientInitSession
	KClientGetTicketForService
	KClientLogin
	KClientPasswordLogin
	KClientKeyLogin
	KClientLogout
	KClientStatus
	KClientGetUserName
	KClientSetUserName
	KClientCacheInitialTicket
	KClientGetSessionKey
	KClientMakeSendAuth
	KClientVerifySendAuth
	KClientEncrypt
	KClientDecrypt
	KClientErrorText
	SendTicketForService
