LIBRARY	des BASE=0x06000000
EXPORTS
	des_ecb3_encrypt
	des_cbc_cksum
	des_cbc_encrypt
	des_ncbc_encrypt
	des_3cbc_encrypt
	des_cfb_encrypt
	des_ede3_cfb64_encrypt
	des_ede3_ofb64_encrypt
	des_ecb_encrypt
	des_encrypt
	des_encrypt2
	des_ede3_cbc_encrypt
	des_enc_read
	des_enc_write
	crypt
	des_ofb_encrypt
	des_pcbc_encrypt
	des_quad_cksum
	des_read_password
	des_read_2passwords
	des_read_pw_string
	des_set_odd_parity
	des_is_weak_key
	des_set_key
	des_key_sched
	des_string_to_key
	des_string_to_2keys
	des_cfb64_encrypt
	des_ofb64_encrypt
	des_cblock_print_file
	des_new_random_key
	des_init_random_number_generator
	des_set_random_generator_seed
	des_set_sequence_number
	des_generate_random_block
