//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by KrbManager.rc
//
#define IDD_ABOUTBOX                    100
#define IDR_MAINFRAME                   128
#define IDR_KRBMANTYPE                  129
#define IDB_BITMAP1                     134
#define ID_FILE_DESTROY                 32774
#define ID_FILE_LOGIN                   32775

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_3D_CONTROLS                     1
#define _APS_NEXT_RESOURCE_VALUE        136
#define _APS_NEXT_COMMAND_VALUE         32776
#define _APS_NEXT_CONTROL_VALUE         1000
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
