LIBRARY roken BASE=0x68f0000
EXPORTS
	gettimeofday
	strcasecmp
	strtok_r
	snprintf
	asprintf
	vsnprintf
	base64_decode
	base64_encode
	roken_concat
	roken_vconcat
	roken_vmconcat
	roken_mconcat
	getuid
	dns_free_data
	dns_lookup
