/*
 * Copyright (c) 1989 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 * static char copyright[] = "Copyright (c) 1990 Regents of the University of California.\nAll rights reserved.\n";
 * static char SccsId[] = "@(#)@(#)version.h	2.6  2.6 4/3/91";
 *
 */

/* $Id: version.h,v 1.5 1997/08/08 22:50:13 assar Exp $ */

/*
 *  Current version of this POP implementation
 */

#if 0
#define VERSION         krb4_version
#endif
