// KrbManager.h : main header file for the KRBMANAGER application
//

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// CKrbManagerApp:
// See KrbManager.cpp for the implementation of this class
//

class CKrbManagerApp : public CWinApp
{
public:
	CKrbManagerApp();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CKrbManagerApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation
private:
	// Restrict KrbManager to a single instance
	CMutex* m_pmutex;

	//{{AFX_MSG(CKrbManagerApp)
	afx_msg void OnAppAbout();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
