// Doc.cpp : implementation of the CKrbManagerDoc class
//

#include "stdafx.h"
#include "KrbManager.h"

#include "Doc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKrbManagerDoc

IMPLEMENT_DYNCREATE(CKrbManagerDoc, CDocument)

BEGIN_MESSAGE_MAP(CKrbManagerDoc, CDocument)
	//{{AFX_MSG_MAP(CKrbManagerDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKrbManagerDoc construction/destruction

CKrbManagerDoc::CKrbManagerDoc()
{
	// TODO: add one-time construction code here

}

CKrbManagerDoc::~CKrbManagerDoc()
{
}

BOOL CKrbManagerDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CKrbManagerDoc serialization

void CKrbManagerDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CKrbManagerDoc diagnostics

#ifdef _DEBUG
void CKrbManagerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CKrbManagerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CKrbManagerDoc commands
